#!/usr/bin/env python
# _*_ coding: utf-8 _*_

import sys, os, re
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
from PyQt5.QtWidgets import (QApplication, QWidget, QPushButton, QFileDialog, QLabel, QHBoxLayout, QGroupBox, QTextEdit,
                             QVBoxLayout, QSplitter, QTableWidget, QTabWidget,
                             QTableWidgetItem, QMessageBox, QFormLayout, QRadioButton,
                             QHeaderView,
                             QAbstractItemView)
from PyQt5.QtGui import QIcon, QFont
from PyQt5.QtCore import Qt, pyqtSignal
from util import PlotWidgets
import numpy as np
import pandas as pd
import torch
from util.EvaluationMetrics import Metrics
from torch.utils.data import DataLoader
from util.Nets import (DealDataset, Net_CNN_1, Net_CNN_11, Net_RNN_2, Net_ABCNN_4, Net_ResNet_5, Net_AutoEncoder_6)
import qdarkstyle
import sip
import joblib

class iLearnPlusLoadModel(QWidget):
    close_signal = pyqtSignal(str)
    def __init__(self):
        super(iLearnPlusLoadModel, self).__init__()

        """ Machine Learning Variable """
        self.data_index = {
            'Training_data': None,
            'Testing_data': None,
            'Training_score': None,
            'Testing_score': None,
            'Metrics': None,
            'ROC': None,
            'PRC': None,
            'Model': None,
        }
        self.current_data_index = 0
        self.ml_running_status = False

        self.model_list = []
        self.dataframe = None
        self.datalabel = None
        self.score = None
        self.metrics = None
        self.aucData = None
        self.prcData = None

        # initialize UI
        self.initUI()

    def initUI(self):
        self.setWindowTitle('iLearnPlus LoadModel')
        self.resize(800, 600)
        self.setWindowState(Qt.WindowMaximized)
        self.setWindowIcon(QIcon(os.path.join(pPath, 'images', 'logo.ico')))

        # file
        topGroupBox = QGroupBox('Load data', self)
        topGroupBox.setFont(QFont('Arial', 10))
        topGroupBox.setMinimumHeight(100)
        topGroupBoxLayout = QFormLayout()
        modelFileButton = QPushButton('Load')
        modelFileButton.setToolTip('One or more models could be loaded.')
        modelFileButton.clicked.connect(self.loadModel)
        testFileButton = QPushButton('Open')
        testFileButton.clicked.connect(self.loadDataFile)
        topGroupBoxLayout.addRow('Open model file(s):', modelFileButton)
        topGroupBoxLayout.addRow('Open testing file:', testFileButton)
        topGroupBox.setLayout(topGroupBoxLayout)

        # start button
        startGroupBox = QGroupBox('Operator', self)
        startGroupBox.setFont(QFont('Arial', 10))
        startLayout = QHBoxLayout(startGroupBox)
        self.ml_start_button = QPushButton('Start')
        self.ml_start_button.clicked.connect(self.run_model)
        self.ml_start_button.setFont(QFont('Arial', 10))
        self.ml_save_button = QPushButton('Save')
        self.ml_save_button.setFont(QFont('Arial', 10))
        self.ml_save_button.clicked.connect(self.save_ml_files)
        startLayout.addWidget(self.ml_start_button)
        startLayout.addWidget(self.ml_save_button)

        # log
        logGroupBox = QGroupBox('Log', self)
        logGroupBox.setFont(QFont('Arial', 10))
        logLayout = QHBoxLayout(logGroupBox)
        self.logTextEdit = QTextEdit()
        self.logTextEdit.setFont(QFont('Arial', 8))
        logLayout.addWidget(self.logTextEdit)


        ### layout
        left_vertical_layout = QVBoxLayout()
        left_vertical_layout.addWidget(topGroupBox)
        left_vertical_layout.addWidget(logGroupBox)
        left_vertical_layout.addWidget(startGroupBox)

        #### widget
        leftWidget = QWidget()
        leftWidget.setLayout(left_vertical_layout)

        #### view region
        scoreTabWidget = QTabWidget()
        trainScoreWidget = QWidget()
        scoreTabWidget.setFont(QFont('Arial', 8))
        scoreTabWidget.addTab(trainScoreWidget, 'Prediction score and evaluation metrics')
        train_score_layout = QVBoxLayout(trainScoreWidget)
        self.train_score_tableWidget = QTableWidget()
        self.train_score_tableWidget.setFont(QFont('Arial', 8))
        self.train_score_tableWidget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.train_score_tableWidget.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        train_score_layout.addWidget(self.train_score_tableWidget)

        self.metricsTableWidget = QTableWidget()
        self.metricsTableWidget.setFont(QFont('Arial', 8))
        self.metricsTableWidget.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        self.metricsTableWidget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.metricsTableWidget.resizeRowsToContents()
        splitter_middle = QSplitter(Qt.Vertical)
        splitter_middle.addWidget(scoreTabWidget)
        splitter_middle.addWidget(self.metricsTableWidget)

        self.dataTableWidget = QTableWidget(2, 4)
        self.dataTableWidget.setFont(QFont('Arial', 8))
        self.dataTableWidget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.dataTableWidget.setShowGrid(False)
        self.dataTableWidget.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        self.dataTableWidget.horizontalHeader().setSectionResizeMode(0, QHeaderView.ResizeToContents)
        self.dataTableWidget.horizontalHeader().setSectionResizeMode(3, QHeaderView.ResizeToContents)
        self.dataTableWidget.setHorizontalHeaderLabels(['Select', 'Data', 'Shape', 'Source'])
        self.dataTableWidget.verticalHeader().setVisible(False)

        self.roc_curve_widget = PlotWidgets.CurveWidget()
        self.prc_curve_widget = PlotWidgets.CurveWidget()
        plotTabWidget = QTabWidget()
        plotTabWidget.setFont(QFont('Arial', 8))
        rocWidget = QWidget()
        self.rocLayout = QVBoxLayout(rocWidget)
        self.rocLayout.addWidget(self.roc_curve_widget)
        prcWidget = QWidget()
        self.prcLayout = QHBoxLayout(prcWidget)
        self.prcLayout.addWidget(self.prc_curve_widget)
        plotTabWidget.addTab(rocWidget, 'ROC curve')
        plotTabWidget.addTab(prcWidget, 'PRC curve')
        splitter_right = QSplitter(Qt.Vertical)
        splitter_right.addWidget(self.dataTableWidget)
        splitter_right.addWidget(plotTabWidget)
        splitter_right.setSizes([100, 300])

        splitter_view = QSplitter(Qt.Horizontal)
        splitter_view.addWidget(splitter_middle)
        splitter_view.addWidget(splitter_right)
        splitter_view.setSizes([100, 200])

        ##### splitter
        splitter_1 = QSplitter(Qt.Horizontal)
        splitter_1.addWidget(leftWidget)
        splitter_1.addWidget(splitter_view)
        splitter_1.setSizes([100, 1200])

        ###### vertical layout
        vLayout = QVBoxLayout()

        ## status bar
        statusGroupBox = QGroupBox('Status', self)
        statusGroupBox.setFont(QFont('Arial', 10))
        statusLayout = QHBoxLayout(statusGroupBox)
        self.ml_status_label = QLabel('Welcome to iLearnPlus LoadModel')
        self.ml_progress_bar = QLabel()
        self.ml_progress_bar.setMaximumWidth(230)
        statusLayout.addWidget(self.ml_status_label)
        statusLayout.addWidget(self.ml_progress_bar)

        splitter_2 = QSplitter(Qt.Vertical)
        splitter_2.addWidget(splitter_1)
        splitter_2.addWidget(statusGroupBox)
        splitter_2.setSizes([1000, 100])
        vLayout.addWidget(splitter_2)
        self.setLayout(vLayout)

    def loadModel(self):
        model_files, ok = QFileDialog.getOpenFileNames(self, 'Open', os.path.join(pPath, 'data'), 'PKL Files (*.pkl)')
        if len(model_files) > 0:
            self.model_list = []
            for file in model_files:
                error_tag_ml = False
                error_tag_dl = False
                model = None
                try:
                    model = joblib.load(file)
                except Exception as e:
                    error_tag_ml = True
                
                if error_tag_ml:
                    try:
                        model = torch.load(file)
                    except Exception as e:
                        error_tag_dl = True
                else:
                    if 'predict_proba' not in dir(model):
                        try:
                            model = torch.load(file)
                        except Exception as e:
                            error_tag_dl = True

                if not error_tag_dl or not error_tag_ml:
                    self.model_list.append(model)
            if len(self.model_list) > 0:                
                self.logTextEdit.append('Load model successfully.')
                self.logTextEdit.append('Model number: %s' %len(model_files))
                return True
            else:
                QMessageBox.critical(self, 'Error', 'Load model failed.', QMessageBox.Ok | QMessageBox.No, QMessageBox.Ok)
                self.model_list = []
                return False
        else:
            return False

    def loadDataFile(self):
        file, ok = QFileDialog.getOpenFileName(self, 'Open', os.path.join(pPath, 'data'), 'CSV Files (*.csv);;TSV Files (*.tsv);;SVM Files(*.svm);;Weka Files (*.arff)')
        if ok:
            if not os.path.exists(file):
                QMessageBox.critical(self, 'Error', 'Data file does not exist.', QMessageBox.Ok | QMessageBox.No, QMessageBox.Ok)
                return False
            self.dataframe, self.datalabel = None, None
            try:
                if file.endswith('.tsv'):
                    df = pd.read_csv(file, sep='\t', header=None)
                    self.dataframe = df.iloc[:, 1:]
                    self.dataframe.index = ['Sample_%s' % i for i in range(dataframe.values.shape[0])]
                    self.dataframe.columns = ['F_%s' % i for i in range(dataframe.values.shape[1])]
                    self.datalabel = np.array(df.iloc[:, 0]).astype(int)
                elif file.endswith('.csv'):
                    df = pd.read_csv(file, sep=',', header=None)
                    self.dataframe = df.iloc[:, 1:]
                    self.dataframe.index = ['Sample_%s' % i for i in range(self.dataframe.values.shape[0])]
                    self.dataframe.columns = ['F_%s' % i for i in range(self.dataframe.values.shape[1])]
                    self.datalabel = np.array(df.iloc[:, 0]).astype(int)
                elif file.endswith('.svm'):
                    with open(file) as f:
                        record = f.read().strip()
                    record = re.sub('\d+:', '', record)
                    array = np.array([[i for i in item.split()] for item in record.split('\n')])
                    self.dataframe = pd.DataFrame(array[:, 1:], dtype=float)
                    self.dataframe.index = ['Sample_%s' % i for i in range(self.dataframe.values.shape[0])]
                    self.dataframe.columns = ['F_%s' % i for i in range(self.dataframe.values.shape[1])]
                    self.datalabel = array[:, 0].astype(int)
                else:
                    with open(file) as f:
                        record = f.read().strip().split('@')[-1].split('\n')[1:]
                    array = np.array([item.split(',') for item in record])
                    self.dataframe = pd.DataFrame(array[:, 0:-1], dtype=float)
                    self.dataframe.index = ['Sample_%s' % i for i in range(self.dataframe.values.shape[0])]
                    self.dataframe.columns = ['F_%s' % i for i in range(self.dataframe.values.shape[1])]
                    label = []
                    for i in array[:, -1]:
                        if i == 'yes':
                            label.append(1)
                        else:
                            label.append(0)
                    self.datalabel = np.array(label)

            except Exception as e:
                QMessageBox.critical(self, 'Error', 'Open data file failed.', QMessageBox.Ok | QMessageBox.No, QMessageBox.Ok)
                return False
            self.logTextEdit.append('Load data file successfully.')
            self.logTextEdit.append('Data shape: %s' %(str(self.dataframe.values.shape)))
            return True
        else:
            return False

    def run_model(self):
        # reset
        self.score = None
        self.metrics = None

        if len(self.model_list) > 0 and not self.dataframe is None:
            try:
                prediction_score = None
                for model in self.model_list:
                    if 'predict_proba' not in dir(model):
                        valid_set = DealDataset(self.dataframe.values, self.datalabel.reshape((-1, 1)))
                        valid_loader = DataLoader(valid_set, batch_size=512, shuffle=False)
                        tmp_prediction_score = model.predict(valid_loader)
                    else:
                        tmp_prediction_score = model.predict_proba(self.dataframe.values)

                    if prediction_score is None:
                        prediction_score = tmp_prediction_score
                    else:
                        prediction_score += tmp_prediction_score
                prediction_score /= len(self.model_list)
                self.score = prediction_score

                # display prediction score
                if not self.score is None:
                    data = self.score
                    self.train_score_tableWidget.setRowCount(data.shape[0])
                    self.train_score_tableWidget.setColumnCount(data.shape[1])
                    self.train_score_tableWidget.setHorizontalHeaderLabels(['Score for category %s' %i for i in range(data.shape[1])])
                    for i in range(data.shape[0]):
                        for j in range(data.shape[1]):
                            cell = QTableWidgetItem(str(round(data[i][j], 4)))
                            self.train_score_tableWidget.setItem(i, j, cell)
                    if self.data_index['Training_score'] is None:
                        # index = self.current_data_index
                        index = 0
                        self.data_index['Training_score'] = index
                        self.dataTableWidget.insertRow(index)
                        self.current_data_index += 1
                    else:
                        # index = self.data_index['Training_score']
                        index = 0
                    self.training_score_radio = QRadioButton()
                    self.dataTableWidget.setCellWidget(index, 0, self.training_score_radio)
                    self.dataTableWidget.setItem(index, 1, QTableWidgetItem('Training score'))
                    self.dataTableWidget.setItem(index, 2, QTableWidgetItem(str(data.shape)))
                    self.dataTableWidget.setItem(index, 3, QTableWidgetItem('NA'))

                # calculate and display evaluation metrics
                column_name = ['Sn', 'Sp', 'Pre', 'Acc', 'MCC', 'F1', 'AUROC', 'AUPRC']
                if not self.score is None and self.score.shape[1] == 2:
                    # calculate metrics
                    data = self.score
                    metrics = Metrics(data[:, -1], self.datalabel)
                    metrics_ind = np.array(
                        [metrics.sensitivity, metrics.specificity, metrics.precision, metrics.accuracy, metrics.mcc,
                         metrics.f1, metrics.auc, metrics.prc]).reshape((1, -1))
                    index_name = ['Metrics value']
                    self.aucData = ['AUROC = %s' % metrics.auc, metrics.aucDot]
                    self.prcData = ['AUPRC = %s' % metrics.prc, metrics.prcDot]
                    del metrics
                    self.metrics = pd.DataFrame(metrics_ind, index=index_name, columns=column_name)

                    # display metrics
                    data = self.metrics.values
                    self.metricsTableWidget.setRowCount(data.shape[0])
                    self.metricsTableWidget.setColumnCount(data.shape[1])
                    self.metricsTableWidget.setHorizontalHeaderLabels(
                        ['Sn (%)', 'Sp (%)', 'Pre (%)', 'Acc (%)', 'MCC', 'F1', 'AUROC', 'AUPRC'])
                    self.metricsTableWidget.setVerticalHeaderLabels(self.metrics.index)
                    for i in range(data.shape[0]):
                        for j in range(data.shape[1]):
                            cell = QTableWidgetItem(str(data[i][j]))
                            self.metricsTableWidget.setItem(i, j, cell)
                    if self.data_index['Metrics'] is None:
                        # index = self.current_data_index
                        index = 1
                        self.data_index['Metrics'] = index
                        self.dataTableWidget.insertRow(index)
                        self.current_data_index += 1
                    else:
                        # index = self.data_index['Metrics']
                        index = 1
                    self.metrics_radio = QRadioButton()
                    self.dataTableWidget.setCellWidget(index, 0, self.metrics_radio)
                    self.dataTableWidget.setItem(index, 1, QTableWidgetItem('Evaluation metrics'))
                    self.dataTableWidget.setItem(index, 2, QTableWidgetItem(str(data.shape)))
                    self.dataTableWidget.setItem(index, 3, QTableWidgetItem('NA'))

                #plot ROC
                if not self.aucData is None:
                    self.rocLayout.removeWidget(self.roc_curve_widget)
                    sip.delete(self.roc_curve_widget)
                    self.roc_curve_widget = PlotWidgets.CurveWidget()
                    self.roc_curve_widget.init_data(0, 'ROC curve', ind_data=self.aucData)
                    self.rocLayout.addWidget(self.roc_curve_widget)

                # plot PRC
                if not self.prcData is None:
                    self.prcLayout.removeWidget(self.prc_curve_widget)
                    sip.delete(self.prc_curve_widget)
                    self.prc_curve_widget = PlotWidgets.CurveWidget()
                    self.prc_curve_widget.init_data(1, 'PRC curve', ind_data=self.prcData)
                    self.prcLayout.addWidget(self.prc_curve_widget)
            except Exception as e:
                QMessageBox.critical(self, 'Error', str(e), QMessageBox.Ok | QMessageBox.No, QMessageBox.Ok)
        else:
            QMessageBox.critical(self, 'Error', 'Please load the model file(s) or data file.', QMessageBox.Ok | QMessageBox.No,
                                 QMessageBox.Ok)

    def save_ml_files(self):
        tag = 0
        try:
            if self.training_score_radio.isChecked():
                tag = 1
                save_file, ok = QFileDialog.getSaveFileName(self, 'Save', './data', 'TSV Files (*.tsv)')
                if ok:
                    ok1 = self.save_prediction_score(save_file)                    
                    if not ok1:
                        QMessageBox.critical(self, 'Error', 'Save file failed.', QMessageBox.Ok | QMessageBox.No,
                                            QMessageBox.Ok)
        except Exception as e:
            pass

        try:
            if self.metrics_radio.isChecked():
                tag = 1
                save_file, ok = QFileDialog.getSaveFileName(self, 'Save', './data', 'TSV Files (*.tsv)')
                if ok:
                    ok1 = self.save_metrics(save_file)                    
                    if not ok1:
                        QMessageBox.critical(self, 'Error', 'Save file failed.', QMessageBox.Ok | QMessageBox.No,
                                            QMessageBox.Ok)
        except Exception as e:
            pass

        if tag == 0:
            QMessageBox.critical(self, 'Error', 'Please select which data to save.', QMessageBox.Ok | QMessageBox.No,
                                 QMessageBox.Ok)

    def save_prediction_score(self, file):
        try:
            df = pd.DataFrame(self.score, columns=['Score_%s' %i for i in range(self.score.shape[1])])
            df.to_csv(file, sep='\t', header=True, index=False)
            return True
        except Exception as e:
            print(e)
            return False

    def save_metrics(self, file):
        try:
            self.metrics.to_csv(file, sep='\t', header=True, index=True)
            return True
        except Exception as e:
            return False

    def closeEvent(self, event):
        reply = QMessageBox.question(self, 'Confirm Exit', 'Are you sure want to quit iLearnPlus?', QMessageBox.Yes | QMessageBox.No,
                                     QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.close_signal.emit('LoadModel')
            self.close()
        else:
            if event:
                event.ignore()

if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = iLearnPlusLoadModel()
    window.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
    window.show()
    sys.exit(app.exec_())


#!/usr/bin/env python
# _*_ coding: utf-8 _*_

import pickle

with open('AAindex.txt') as f:
    records = f.readlines()[1:]

my_dict = {}

for line in records:
    array = line.strip().split('\t') if line.strip() != '' else None
    my_dict[array[0]] = [float(array[i]) for i in range(1, len(array))]

with open('AAindex.data', 'wb') as f:
    pickle.dump(my_dict, f)

with open('AAindex.data', 'rb') as f:
    tmp = pickle.load(f)
print(tmp)


#!/usr/bin/env python
# _*_ coding: utf-8 _*_

import sys, os, platform
import pickle

didna_list = ['Base stacking', 'Protein induced deformability', 'B-DNA twist', 'Dinucleotide GC Content', 'A-philicity',
              'Propeller twist', 'Duplex stability:(freeenergy)',
              'Duplex tability(disruptenergy)', 'DNA denaturation', 'Bending stiffness', 'Protein DNA twist',
              'Stabilising energy of Z-DNA', 'Aida_BA_transition', 'Breslauer_dG', 'Breslauer_dH',
              'Breslauer_dS', 'Electron_interaction', 'Hartman_trans_free_energy', 'Helix-Coil_transition',
              'Ivanov_BA_transition', 'Lisser_BZ_transition', 'Polar_interaction', 'SantaLucia_dG',
              'SantaLucia_dH', 'SantaLucia_dS', 'Sarai_flexibility', 'Stability', 'Stacking_energy',
              'Sugimoto_dG', 'Sugimoto_dH', 'Sugimoto_dS', 'Watson-Crick_interaction', 'Twist', 'Tilt', 'Roll',
              'Shift', 'Slide', 'Rise',
              'Clash Strength', 'Roll_roll', 'Twist stiffness', 'Tilt stiffness', 'Shift_rise',
              'Adenine content', 'Direction', 'Twist_shift', 'Enthalpy1', 'Twist_twist', 'Roll_shift',
              'Shift_slide', 'Shift2', 'Tilt3', 'Tilt1', 'Tilt4', 'Tilt2', 'Slide (DNA-protein complex)1',
              'Tilt_shift', 'Twist_tilt', 'Twist (DNA-protein complex)1', 'Tilt_rise', 'Roll_rise',
              'Stacking energy', 'Stacking energy1', 'Stacking energy2', 'Stacking energy3', 'Propeller Twist',
              'Roll11', 'Rise (DNA-protein complex)', 'Tilt_tilt', 'Roll4', 'Roll2', 'Roll3', 'Roll1',
              'Minor Groove Size', 'GC content', 'Slide_slide', 'Enthalpy', 'Shift_shift', 'Slide stiffness',
              'Melting Temperature1', 'Flexibility_slide', 'Minor Groove Distance',
              'Rise (DNA-protein complex)1', 'Tilt (DNA-protein complex)', 'Guanine content',
              'Roll (DNA-protein complex)1', 'Entropy', 'Cytosine content', 'Major Groove Size', 'Twist_rise',
              'Major Groove Distance', 'Twist (DNA-protein complex)', 'Purine (AG) content',
              'Melting Temperature', 'Free energy', 'Tilt_slide', 'Major Groove Width', 'Major Groove Depth',
              'Wedge', 'Free energy8', 'Free energy6', 'Free energy7', 'Free energy4', 'Free energy5',
              'Free energy2', 'Free energy3', 'Free energy1', 'Twist_roll', 'Shift (DNA-protein complex)',
              'Rise_rise', 'Flexibility_shift', 'Shift (DNA-protein complex)1', 'Thymine content', 'Slide_rise',
              'Tilt_roll', 'Tip', 'Keto (GT) content', 'Roll stiffness', 'Minor Groove Width', 'Inclination',
              'Entropy1', 'Roll_slide', 'Slide (DNA-protein complex)', 'Twist1', 'Twist3', 'Twist2', 'Twist5',
              'Twist4', 'Twist7', 'Twist6', 'Tilt (DNA-protein complex)1', 'Twist_slide', 'Minor Groove Depth',
              'Roll (DNA-protein complex)', 'Rise2', 'Persistance Length', 'Rise3', 'Shift stiffness',
              'Probability contacting nucleosome core', 'Mobility to bend towards major groove', 'Slide3',
              'Slide2', 'Slide1', 'Shift1', 'Bend', 'Rise1', 'Rise stiffness',
              'Mobility to bend towards minor groove']
tridna_list = ['Dnase I', 'Bendability (DNAse)', 'Bendability (consensus)', 'Trinucleotide GC Content',
               'Nucleosome positioning', 'Consensus_roll', 'Consensus-Rigid', 'Dnase I-Rigid', 'MW-Daltons',
               'MW-kg', 'Nucleosome', 'Nucleosome-Rigid']
dirna_list = ['Slide (RNA)', 'Adenine content', 'Hydrophilicity (RNA)', 'Tilt (RNA)', 'Stacking energy (RNA)',
              'Twist (RNA)', 'Entropy (RNA)', 'Roll (RNA)', 'Purine (AG) content', 'Hydrophilicity (RNA)1',
              'Enthalpy (RNA)1', 'GC content', 'Entropy (RNA)1', 'Rise (RNA)', 'Free energy (RNA)',
              'Keto (GT) content', 'Free energy (RNA)1', 'Enthalpy (RNA)', 'Guanine content', 'Shift (RNA)',
              'Cytosine content', 'Thymine content']
myDict = {
    'DAC': {'DNA': didna_list, 'RNA': dirna_list},
    'DCC': {'DNA': didna_list, 'RNA': dirna_list},
    'DACC': {'DNA': didna_list, 'RNA': dirna_list},
    'TAC': {'DNA': tridna_list, 'RNA': []},
    'TCC': {'DNA': tridna_list, 'RNA': []},
    'TACC': {'DNA': tridna_list, 'RNA': []},
    'PseDNC': {'DNA': didna_list, 'RNA': dirna_list},
    'PseKNC': {'DNA': didna_list, 'RNA': dirna_list},
    'PCPseDNC': {'DNA': didna_list, 'RNA': dirna_list},
    'PCPseTNC': {'DNA': tridna_list, 'RNA': []},
    'SCPseDNC': {'DNA': didna_list, 'RNA': dirna_list},
    'SCPseTNC': {'DNA': tridna_list, 'RNA': []},
}
myDictDefault = {
    'DAC': {'DNA': ['Rise', 'Roll', 'Shift', 'Slide', 'Tilt', 'Twist'],
            'RNA': ['Rise (RNA)', 'Roll (RNA)', 'Shift (RNA)', 'Slide (RNA)', 'Tilt (RNA)', 'Twist (RNA)']},
    'DCC': {'DNA': ['Rise', 'Roll', 'Shift', 'Slide', 'Tilt', 'Twist'],
            'RNA': ['Rise (RNA)', 'Roll (RNA)', 'Shift (RNA)', 'Slide (RNA)', 'Tilt (RNA)', 'Twist (RNA)']},
    'DACC': {'DNA': ['Rise', 'Roll', 'Shift', 'Slide', 'Tilt', 'Twist'],
             'RNA': ['Rise (RNA)', 'Roll (RNA)', 'Shift (RNA)', 'Slide (RNA)', 'Tilt (RNA)', 'Twist (RNA)']},
    'TAC': {'DNA': ['Dnase I', 'Bendability (DNAse)'], 'RNA': []},
    'TCC': {'DNA': ['Dnase I', 'Bendability (DNAse)'], 'RNA': []},
    'TACC': {'DNA': ['Dnase I', 'Bendability (DNAse)'], 'RNA': []},
    'PseDNC': {'DNA': ['Rise', 'Roll', 'Shift', 'Slide', 'Tilt', 'Twist'],
               'RNA': ['Rise (RNA)', 'Roll (RNA)', 'Shift (RNA)', 'Slide (RNA)', 'Tilt (RNA)', 'Twist (RNA)']},
    'PseKNC': {'DNA': ['Rise', 'Roll', 'Shift', 'Slide', 'Tilt', 'Twist'],
               'RNA': ['Rise (RNA)', 'Roll (RNA)', 'Shift (RNA)', 'Slide (RNA)', 'Tilt (RNA)', 'Twist (RNA)']},
    'PCPseDNC': {
        'DNA': ['Base stacking', 'Protein induced deformability', 'B-DNA twist', 'A-philicity', 'Propeller twist',
                'Duplex stability:(freeenergy)', 'DNA denaturation', 'Bending stiffness', 'Protein DNA twist',
                'Aida_BA_transition', 'Breslauer_dG', 'Breslauer_dH', 'Electron_interaction',
                'Hartman_trans_free_energy', 'Helix-Coil_transition', 'Lisser_BZ_transition', 'Polar_interaction',
                'SantaLucia_dG', 'SantaLucia_dS', 'Sarai_flexibility', 'Stability', 'Sugimoto_dG', 'Sugimoto_dH',
                'Sugimoto_dS', 'Duplex tability(disruptenergy)', 'Stabilising energy of Z-DNA', 'Breslauer_dS',
                'Ivanov_BA_transition', 'SantaLucia_dH', 'Stacking_energy', 'Watson-Crick_interaction',
                'Dinucleotide GC Content', 'Twist', 'Tilt', 'Roll', 'Shift', 'Slide', 'Rise'],
        'RNA': ['Rise (RNA)', 'Roll (RNA)', 'Shift (RNA)', 'Slide (RNA)', 'Tilt (RNA)', 'Twist (RNA)']},
    'PCPseTNC': {'DNA': ['Dnase I', 'Bendability (DNAse)'], 'RNA': []},
    'SCPseDNC': {'DNA': ['Rise', 'Roll', 'Shift', 'Slide', 'Tilt', 'Twist'],
                 'RNA': ['Rise (RNA)', 'Roll (RNA)', 'Shift (RNA)', 'Slide (RNA)', 'Tilt (RNA)', 'Twist (RNA)']},
    'SCPseTNC': {'DNA': ['Dnase I', 'Bendability (DNAse)'], 'RNA': []},
}
myKmer = {
    'DAC': 2, 'DCC': 2, 'DACC': 2,
    'TAC': 3, 'TCC': 3, 'TACC': 3
}
myDataFile = {
    'DAC': {'DNA': 'didnaPhyche.data', 'RNA': 'dirnaPhyche.data'},
    'DCC': {'DNA': 'didnaPhyche.data', 'RNA': 'dirnaPhyche.data'},
    'DACC': {'DNA': 'didnaPhyche.data', 'RNA': 'dirnaPhyche.data'},
    'TAC': {'DNA': 'tridnaPhyche.data', 'RNA': ''},
    'TCC': {'DNA': 'tridnaPhyche.data', 'RNA': ''},
    'TACC': {'DNA': 'tridnaPhyche.data', 'RNA': ''},
    'PseDNC': {'DNA': 'didnaPhyche.data', 'RNA': 'dirnaPhyche.data'},
    'PseKNC': {'DNA': 'didnaPhyche.data', 'RNA': 'dirnaPhyche.data'},
    'PCPseDNC': {'DNA': 'didnaPhyche.data', 'RNA': 'dirnaPhyche.data'},
    'PCPseTNC': {'DNA': 'tridnaPhyche.data', 'RNA': ''},
    'SCPseDNC': {'DNA': 'didnaPhyche.data', 'RNA': 'dirnaPhyche.data'},
    'SCPseTNC': {'DNA': 'tridnaPhyche.data', 'RNA': ''},
}

def check_acc_arguments(method, type, parameters):
    """
    Check auto-correlation parameters.
    :param method: i.e. DAC, DCC, DACC, TAC, TCC, TACC
    :param type: i.e. DNA, RNA
    :return:
    """
    kmer = myKmer[method]
    myIndex = []
    myProperty = {}
    dataFile = ''

    if type == 'DNA':
        if method in ('DAC', 'DCC', 'DACC'):
            myIndex = parameters['Di-DNA-Phychem'].strip().split(';')
        else:
            myIndex = parameters['Tri-DNA-Phychem'].strip().split(';')
    if type == 'RNA':
        if method in ('DAC', 'DCC', 'DACC'):
            myIndex = parameters['Di-RNA-Phychem'].strip().split(';')

    dataFile = myDataFile[method][type]
    file_path = os.path.join(os.path.split(os.path.realpath(__file__))[0], 'data', dataFile)
    # file_path = os.path.split(os.path.realpath(__file__))[0] + r'\data\%s'%dataFile if platform.system() == 'Windows' else os.path.split(os.path.realpath(__file__))[0] + r'/data/%s' %dataFile
    with open(file_path, 'rb') as f:
        myProperty = pickle.load(f)
    if len(myIndex) == 0 or len(myProperty) == 0:
        return myIndex, myProperty, kmer, False
    return myIndex, myProperty, kmer, True

def check_Pse_arguments(method, type, parameters):
    """
    Check auto-correlation parameters.
    :param method: i.e. PseDNC, PseKNC, PCPseDNC, PCPseTNC, SCPseDNC, SCPseTNC
    :param type: i.e. DNA, RNA
    :return:
    """
    myIndex = []
    myProperty = {}
    dataFile = ''

    if type == 'DNA':
        if method in ('PseDNC', 'PseKNC'):
            myIndex = parameters['Di-DNA-Phychem'].strip().split(';')
        if method in ('PCPseDNC', 'SCPseDNC'):
            myIndex = parameters['Di-DNA-Phychem'].strip().split(';')
        if method in ('PCPseTNC', 'SCPseTNC'):
            myIndex = parameters['Tri-DNA-Phychem'].strip().split(';')
    if type == 'RNA':
        if method in ('PseDNC', 'PseKNC'):
            myIndex = parameters['Di-RNA-Phychem'].strip().split(';')
        if method in ('PCPseDNC', 'SCPseDNC'):
            myIndex = parameters['Di-RNA-Phychem'].strip().split(';')

    dataFile = myDataFile[method][type]
    file_path = os.path.join(os.path.split(os.path.realpath(__file__))[0], 'data', dataFile)
    # print(file_path)
    # file_path = os.path.split(os.path.realpath(__file__))[0] + r'\data\%s'%dataFile if platform.system() == 'Windows' else os.path.split(os.path.realpath(__file__))[0] + r'/data/%s' %dataFile
    with open(file_path, 'rb') as f:
        myProperty = pickle.load(f)

    if len(myIndex) == 0 or len(myProperty) == 0:
        return myIndex, myProperty, False

    return myIndex, myProperty, True



#!/usr/bin/env python
# _*_ coding: utf-8 _*_

import numpy as np
import pandas as pd

class MarkvCluster(object):
    def __init__(self, data, expand_factor=2, inflate_factor=2.0, mult_factor=2.0, max_loop=200):
        super(MarkvCluster, self).__init__()
        M = np.corrcoef(data)
        M[M < 0] = 0
        for i in range(len(M)):
            M[i][i] = 0
        import networkx as nx
        G = nx.from_numpy_matrix(M)
        self.M, self.clusters = self.networkx_mcl(G, expand_factor=expand_factor, inflate_factor=inflate_factor,
                                                  max_loop=max_loop, mult_factor=mult_factor)
        self.cluster_array = self.get_array()

    def get_array(self):
        array = []
        for key in self.clusters:
            for value in self.clusters[key]:
                array.append([value, key])
        df = pd.DataFrame(np.array(array), columns=['Sample', 'Cluster'])
        df = df.sort_values(by='Sample', ascending=True)
        return df.Cluster.values

    def networkx_mcl(self, G, expand_factor=2, inflate_factor=2, max_loop=10, mult_factor=1):
        import networkx as nx
        A = nx.adjacency_matrix(G)
        return self.mcl(np.array(A.todense()), expand_factor, inflate_factor, max_loop, mult_factor)

    def mcl(self, M, expand_factor=2, inflate_factor=2, max_loop=10, mult_factor=1):
        M = self.add_diag(M, mult_factor)
        M = self.normalize(M)
        for i in range(max_loop):
            # logging.info("loop %s" % i)
            M = self.inflate(M, inflate_factor)
            M = self.expand(M, expand_factor)
            if self.stop(M, i): break

        clusters = self.get_clusters(M)
        return M, clusters

    def add_diag(self, A, mult_factor):
        return A + mult_factor * np.identity(A.shape[0])

    def normalize(self, A):
        column_sums = A.sum(axis=0)
        new_matrix = A / column_sums[np.newaxis, :]
        return new_matrix

    def inflate(self, A, inflate_factor):
        return self.normalize(np.power(A, inflate_factor))

    def expand(self, A, expand_factor):
        return np.linalg.matrix_power(A, expand_factor)

    def stop(self, M, i):
        if i % 5 == 4:
            m = np.max(M ** 2 - M) - np.min(M ** 2 - M)
            if m == 0:
                # logging.info("Stop at iteration %s" % i)
                return True
        return False

    def get_clusters(self, A):
        clusters = []
        for i, r in enumerate((A > 0).tolist()):
            if r[i]:
                clusters.append(A[i, :] > 0)
        clust_map = {}
        for cn, c in enumerate(clusters):
            for x in [i for i, x in enumerate(c) if x]:
                clust_map[cn] = clust_map.get(cn, []) + [x]
        return clust_map


if __name__ == '__main__':
    data = np.random.rand(32, 5)
    print(data)
    mcl = MarkvCluster(data)
    print(mcl.cluster_array)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: Zhen Chen(chenzhen-win2009@163.com)
# Description: iLearnPlus: a comprehensive and automated machine-learning platform for nucleic acid and protein sequence analysis, prediction and visualization

from setuptools import setup, find_packages

setup(
    name = 'ilearnplus',
    version = '1.4',
    keywords='ilearnplus',
    description = 'A comprehensive and automated machine-learning platform for nucleic acid and protein sequence analysis, prediction and visualization',
    license = 'MIT License',
    url = 'https://github.com/Superzchen/iLearnPlus',
    author = 'SuperZhen',
    author_email = 'chenzhen-win2009@163.com',    
    packages=find_packages("src"),
    package_dir = {'':'src'},
    
    package_data = {
        # 任何包中含有.txt文件，都包含它
        '': ['*.txt'],
        # 包含demo包data文件夹中的 *.dat文件
        'ilearnplus': ['data/*.csv', 'data/*tsv', 'data/*txt', 'docs/*.pdf', 'images/*.*', 'models/*.pkl', 'util/data/*.txt',  'util/data/*.data'],
    },
    platforms = 'any',
    install_requires = [
        # 'PyQt5',
        'qdarkstyle',
        'numpy>=1.18.5',
        'pandas>=1.0.5',
        'sip',
        'datetime',
        'scikit-learn>=0.23.1',
        'scipy>=1.5.0',
        'lightgbm',
        'xgboost',
        'matplotlib>=3.1.1',
        'seaborn',
        'joblib',
        # 'multiprocessing',
        ],     
)


#!/usr/bin/env python
# _*_ coding: utf-8 _*_

import sys, os
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
from PyQt5.QtWidgets import (QMainWindow, QWidget, QVBoxLayout, QAction, QMessageBox, QFileDialog)
from PyQt5.QtCore import QUrl
from PyQt5.QtGui import QIcon, QDesktopServices
import PlotWidgets, InputDialog
import threading
import sip
import pandas as pd
import numpy as np


class PlotCurve(QMainWindow):
    def __init__(self, curve='ROC'):
        super(PlotCurve, self).__init__()
        self.initUI()
        self.curve = curve
        self.data = []
        self.raw_data = {}

    def initUI(self):
        bar = self.menuBar()
        file = bar.addMenu('File')
        add = QAction('Add curve', self)
        add.triggered.connect(self.importData)
        comp = QAction('Compare curves', self)
        comp.triggered.connect(self.compareCurves)
        quit = QAction('Exit', self)
        quit.triggered.connect(self.close)
        file.addAction(add)
        file.addAction(comp)
        file.addSeparator()
        file.addAction(quit)

        help = bar.addMenu('Help')
        about = QAction('Document', self)
        about.triggered.connect(self.openDocumentUrl)
        help.addAction(about)

        self.setWindowTitle('iLearnPlus Plot curve')
        self.resize(600, 600)
        self.setWindowIcon(QIcon('images/logo.ico'))
        curveWidget = QWidget()
        self.curveLayout = QVBoxLayout(curveWidget)
        self.curveGraph = PlotWidgets.CustomCurveWidget()
        self.curveLayout.addWidget(self.curveGraph)
        self.setCentralWidget(curveWidget)

    def closeEvent(self, event):
        reply = QMessageBox.question(self, 'Message', 'Are you sure to quit?', QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.close()

    def importData(self):
        auc, dot, color, lineWidth, lineStyle, prefix, raw_data, ok = InputDialog.QPlotInput.getValues(self.curve)
        if not auc is None and not prefix is None and ok:
            self.raw_data[prefix] = raw_data
            if self.curve == 'ROC':
                self.data.append(['%s AUC = %s'%(prefix, auc), dot, lineWidth, lineStyle, color])
            else:
                self.data.append(['%s AUPRC = %s'%(prefix, auc), dot, lineWidth, lineStyle, color])
            self.curveLayout.removeWidget(self.curveGraph)
            sip.delete(self.curveGraph)
            self.curveGraph = PlotWidgets.CustomCurveWidget()
            if self.curve == 'ROC':
                self.curveGraph.init_data(0, 'ROC curve', self.data)
            else:
                self.curveGraph.init_data(1, 'PRC curve', self.data)
            self.curveLayout.addWidget(self.curveGraph)

    def compareCurves(self):
        if len(self.raw_data) >= 2:
            method, bootstrap_n, ok = InputDialog.QStaticsInput.getValues()
            if ok:
                self.subWin = PlotWidgets.BootstrapTestWidget(self.raw_data, bootstrap_n, self.curve)
                self.subWin.setWindowTitle('Calculating p values ... ')
                self.subWin.resize(600, 600)
                t = threading.Thread(target=self.subWin.bootstrapTest)
                t.start()
                self.subWin.show()
        else:
            QMessageBox.critical(self, 'Error', 'Two or more curve could be compared!', QMessageBox.Ok | QMessageBox.No, QMessageBox.Ok)

    def openDocumentUrl(self):
        QDesktopServices.openUrl(QUrl('https://ilearnplus.erc.monash.edu/'))

class ScatterPlot(QMainWindow):
    def __init__(self):
        super(ScatterPlot, self).__init__()
        self.initUI()       

    def initUI(self):
        bar = self.menuBar()
        file = bar.addMenu('File')
        add = QAction('Open file', self)
        add.triggered.connect(self.importData)        
        quit = QAction('Exit', self)
        quit.triggered.connect(self.close)
        file.addAction(add)        
        file.addSeparator()
        file.addAction(quit)       

        self.setWindowTitle('iLearnPlus Scatter Plot')
        self.resize(600, 600)
        self.setWindowIcon(QIcon('images/logo.ico'))
        plotWidget = QWidget()
        self.plotLayout = QVBoxLayout(plotWidget)
        self.plotGraph = PlotWidgets.ClusteringDiagramMatplotlib()
        self.plotLayout.addWidget(self.plotGraph)
        self.setCentralWidget(plotWidget)
    
    def closeEvent(self, event):
        reply = QMessageBox.question(self, 'Message', 'Are you sure to quit?', QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.close()

    def importData(self):
        try:
            file, ok = QFileDialog.getOpenFileName(self, 'Open', './data', 'TSV (*.tsv)')
            if ok:
                data = pd.read_csv(file, delimiter='\t', header=0, dtype=float)                
                category =[int(i) for i in sorted(set(data.iloc[:, 0]))]
                new_data = []
                for c in category:
                    index = np.where(data.values[:,0] == c)[0]
                    new_data.append([c, data.values[index, 1:]])
                self.plotLayout.removeWidget(self.plotGraph)
                sip.delete(self.plotGraph)
                self.plotGraph = PlotWidgets.ClusteringDiagramMatplotlib()
                self.plotGraph.init_data('Scatter Plot', new_data, data.columns[1], data.columns[2])
                self.plotLayout.addWidget(self.plotGraph)
        except Exception as e:
            QMessageBox.critical(self, 'Error', 'Please check your input.', QMessageBox.Ok | QMessageBox.No, QMessageBox.Ok)

class Hist(QMainWindow):
    def __init__(self):
        super(Hist, self).__init__()
        self.initUI()        

    def initUI(self):
        bar = self.menuBar()
        file = bar.addMenu('File')
        add = QAction('Open file', self)
        add.triggered.connect(self.importData)        
        quit = QAction('Exit', self)
        quit.triggered.connect(self.close)
        file.addAction(add)        
        file.addSeparator()
        file.addAction(quit)       

        self.setWindowTitle('iLearnPlus Histogram and Kernel density plot')
        self.resize(600, 600)
        self.setWindowIcon(QIcon('images/logo.ico'))
        plotWidget = QWidget()
        self.plotLayout = QVBoxLayout(plotWidget)
        self.plotGraph = PlotWidgets.HistogramWidget()
        self.plotLayout.addWidget(self.plotGraph)
        self.setCentralWidget(plotWidget)
    
    def closeEvent(self, event):
        reply = QMessageBox.question(self, 'Message', 'Are you sure to quit?', QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.close()

    def importData(self):
        try:
            file, ok = QFileDialog.getOpenFileName(self, 'Open', './data', 'TSV (*.tsv)')
            if ok:
                data = np.loadtxt(file, dtype=float, delimiter='\t')
                fill_zero = np.zeros((data.shape[0], 1))            
                data = np.hstack((np.zeros((data.shape[0], 1)), data))
                self.plotLayout.removeWidget(self.plotGraph)
                sip.delete(self.plotGraph)
                self.plotGraph = PlotWidgets.HistogramWidget()
                self.plotGraph.init_data('Data distribution', data)
                self.plotLayout.addWidget(self.plotGraph)
        except Exception as e:
            QMessageBox.critical(self, 'Error', 'Please check your input.', QMessageBox.Ok | QMessageBox.No, QMessageBox.Ok)








